/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <string.h>
#include <drivers/gpio.h>
#include <nrfx_spim.h>

#define LED_PORT        DT_ALIAS_LED0_GPIOS_CONTROLLER
#define LED             DT_ALIAS_LED0_GPIOS_PIN

/* 1000 msec = 1 sec */
#define SLEEP_TIME	1000

#define NRFX_SPIM_SCK_PIN  3
#define NRFX_SPIM_MOSI_PIN 12
#define NRFX_SPIM_MISO_PIN 13
#define NRFX_SPIM_SS_PIN   29

#define SPI_INSTANCE  1                                           /**< SPI instance index. */
static const nrfx_spim_t spi = NRFX_SPIM_INSTANCE(SPI_INSTANCE);  /**< SPI instance. */

static volatile bool spi_xfer_done = true;  /**< Flag used to indicate that SPI instance completed the transfer. */

#define TEST_STRING "Hello NRF!"
static uint8_t       m_tx_buf[] = TEST_STRING;           /**< TX buffer. */
static uint8_t       m_rx_buf[sizeof(TEST_STRING) + 1];  /**< RX buffer. */
static const uint8_t m_length = sizeof(m_tx_buf);        /**< Transfer length. */

#ifdef NRF91_SERIES
#define SPIM1_IRQn UARTE1_SPIM1_SPIS1_TWIM1_TWIS1_IRQn
#else /* Assume NRF5-series */
#define SPIM1_IRQn SPI1_TWI1_IRQn
#endif
#define ISR_PRIORITY 5

void spim_event_handler(nrfx_spim_evt_t const * p_event,
                       void *                  p_context)
{
    spi_xfer_done = true;
    printk("Transfer completed\n");
    if (m_rx_buf[0] != 0)
    {
        printk("Received in hex: ");
		for (int i = 0; i < strlen((const char *)m_rx_buf); i++)
			printk("%x ", m_rx_buf[i]);
		printk("\n");
		printk("ascii: %s\n", m_rx_buf);
    }	
}

void setup_spim(void)
{
    nrfx_spim_config_t spi_config = NRFX_SPIM_DEFAULT_CONFIG(NRFX_SPIM_SCK_PIN, NRFX_SPIM_MOSI_PIN, NRFX_SPIM_MISO_PIN, NRFX_SPIM_SS_PIN);

	u32_t err = nrfx_spim_init(&spi, &spi_config, spim_event_handler, NULL);
	if (err != NRFX_SUCCESS) {
		printk("nrfx err: %d\n", err);
		return;
	}
	/* Zephyr IRQ wrapper, corresponds to NVIC_* functions */
	IRQ_CONNECT(SPIM1_IRQn, ISR_PRIORITY, nrfx_spim_1_irq_handler, NULL, 0);
}

void main(void)
{
	u32_t cnt = 0;
	struct device *dev;

	dev = device_get_binding(LED_PORT);
	/* Set LED pin as output */
	gpio_pin_configure(dev, LED, GPIO_DIR_OUT);
	setup_spim();

	printk("NRFX SPIM sample\n");
	printk("Loopback MOSI and MISO to read out the transferred data\n");

	while (1) {
		/* Set pin to HIGH/LOW every 1 second */
		gpio_pin_write(dev, LED, cnt % 2);
		cnt++;
		k_sleep(SLEEP_TIME);

		nrfx_spim_xfer_desc_t xfer_desc = NRFX_SPIM_XFER_TRX(m_tx_buf, m_length, m_rx_buf, m_length);
		if (spi_xfer_done)
		{
			u32_t ret = nrfx_spim_xfer(&spi, &xfer_desc, 0);
			if (ret == NRFX_SUCCESS) {
				spi_xfer_done = false;
			}
		}
	}
}
